# coding=utf-8
import abc
from configparser import ConfigParser
# from typing import List, Dict

import yaml


# Context = Dict[str, 'Context'] | List['Context'] | str | int | float | bool | None


# noinspection PyUnusedLocal
class StrLoader(object):
    @abc.abstractmethod
    def load(self, content: str):
        return {}


class YamlLoader(StrLoader):
    # noinspection PyMethodMayBeStatic
    def load(self, content: str):
        return yaml.safe_load(content)


class IniLoader(StrLoader):
    def __init__(self):
        self._parser = ConfigParser()

    def load(self, content: str):
        self._parser.read_string(content)
        return {
            section: dict(self._parser.items(section))
            for section in self._parser.sections()
        }



import os
from pathlib import Path

DEFAULT_URL_ENVVAR = "CONF_URL"
DISABLE_RELOAD = -1
# 以下目录 `~/.config` 遵从 XDG Base Directory Specification
# 参考: https://specifications.freedesktop.org/basedir-spec/basedir-spec-latest.html
XDG_CONFIG_HOME = Path(os.getenv('XDG_CONFIG_HOME') or '~/.config').expanduser()


def find_config(app_name: str = 'app', filename: str = None):
    if not filename:
        filename = app_name
    default_config_paths = [
        Path.cwd() / filename,                  # ./config.ini
        XDG_CONFIG_HOME / app_name / filename,  # ~/.config/kebab/config.ini
        Path('/etc') / app_name / filename      # /etc/kebab/config.ini
    ]

    for default in default_config_paths:
        if default.exists():
            return default

    return None
